﻿'
' Copyright (c) 2021, MindFusion LLC - Bulgaria.
'

Imports Microsoft.Win32
Imports MindFusion.Diagramming.Wpf.Layout


Class MainWindow

	Public Sub New()

		InitializeComponent()

		overview.Document = diagram

		diagram.DefaultShape = Shapes.Ellipse
		diagram.LinkHeadShape = ArrowHeads.PointerArrow
		diagram.LinkHeadShapeSize = 10

	End Sub

	Private Sub zoomInButton_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
		diagram.ZoomFactor = Math.Min(1000, diagram.ZoomFactor + 10)
	End Sub

	Private Sub zoomOutButton_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
		diagram.ZoomFactor = Math.Max(20, diagram.ZoomFactor - 10)
	End Sub

	Private Sub fitButton_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
		Dim union As Rect = Rect.Empty
		For Each node As DiagramNode In diagram.Nodes
			If union.IsEmpty Then
				union = node.Bounds
			Else
				union.Union(node.Bounds)
			End If
		Next

		union.Inflate(50, 50)

        diagram.ZoomToRect(union)

        border3.BorderBrush = Brushes.Transparent
	End Sub

	Private Sub noZoomButton_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
		diagram.ZoomFactor = 100
	End Sub

	Private Sub generateGraphButton_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
		diagram.ClearAll()

		Dim nodes As Integer = 1000
		Dim links As Integer = 100

		Dim temp As Integer = 0
		If Integer.TryParse(nodesText.Text, temp) Then
			If temp > 0 Then
				nodes = temp
			End If
		End If
		If Integer.TryParse(linksText.Text, temp) Then
			If temp > 0 Then
				links = temp
			End If
		End If

        CreateBlock(nodes, links, BlockFrame.Forest, "")

        border1.BorderBrush = Brushes.Transparent
        border2.BorderBrush = Brushes.Red
	End Sub

	Private Sub layoutButton_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
		' Reset styles
		For Each node As DiagramNode In diagram.Nodes
			node.ClearValue(ShapeNode.BrushProperty)
			node.ClearValue(ShapeNode.StrokeProperty)
		Next

		For Each link As DiagramLink In diagram.Links
			link.ClearValue(DiagramLink.BrushProperty)
			link.ClearValue(DiagramLink.StrokeProperty)
		Next

		ArrangeOnce()

        diagram.ResizeToFitItems(5)

        border2.BorderBrush = Brushes.Transparent
        border3.BorderBrush = Brushes.Red
	End Sub

	Private Sub ArrangeOnce()
		Dim layout As New MindFusion.Diagramming.Wpf.Layout.CompositeLayout()

		layout.RotateSubgraphs = rotateCheckBox.IsChecked = True
		layout.EmphasizeTrees = emphasizeCheckBox.IsChecked = True
		If biconnectedRadioButton.IsChecked = True Then
			layout.PartitionMethod = MindFusion.Layout.PartitionMethod.BiconnectedComponents
		ElseIf shortestPathRadioButton.IsChecked = True Then
			layout.PartitionMethod = MindFusion.Layout.PartitionMethod.ShortestPath
		ElseIf customRadioButton.IsChecked = True Then
			layout.PartitionMethod = MindFusion.Layout.PartitionMethod.[Custom]
		End If

		Dim subtreeLayout As New TreeLayout()
		subtreeLayout.Type = TreeLayoutType.Radial
		subtreeLayout.IgnoreLinkDirection = True
		subtreeLayout.LayoutNode = AddressOf OnSubtreeLayoutNode
		subtreeLayout.LayoutLink = AddressOf OnSubtreeLayoutLink
		layout.SubtreeLayout = subtreeLayout

		If circularRadioButton.IsChecked = True Then
			Dim subgraphLayout As New CircularLayout()
			subgraphLayout.Radius = 0
			subgraphLayout.LayoutNode = AddressOf OnSubgraphLayoutNode
			subgraphLayout.LayoutLink = AddressOf OnSubgraphLayoutLink
			subgraphLayout.SplitGraph = False
			layout.SubgraphLayout = subgraphLayout
		ElseIf layeredRadioButton.IsChecked = True Then
			layout.CustomLayout = AddressOf OnCustomLayeredLayout
		ElseIf annealRadioButton.IsChecked = True Then
			layout.CustomLayout = AddressOf OnCustomAnnealLayout
		ElseIf orthogonalRadioButton.IsChecked = True Then
			layout.CustomLayout = AddressOf OnCustomOrthogonalLayout
		End If

		Dim masterLayout As New AnnealLayout()
		masterLayout.CrossingLinksCost = 0
		masterLayout.LayoutNode = AddressOf OnMasterLayoutNode
		masterLayout.LayoutLink = AddressOf OnMasterLayoutLink
		layout.MasterLayout = masterLayout

		Dim partition As New List(Of IEnumerable)()
		If customRadioButton.IsChecked = True Then
			Dim currentPartition As DiagramNodeCollection = Nothing
			For i As Integer = 0 To diagram.Nodes.Count - 1
				Dim subgraphSize As Integer = 50
				Integer.TryParse(subgraphSizeTextBox.Text, subgraphSize)

				If i Mod subgraphSize = 0 Then
					currentPartition = New DiagramNodeCollection()
					partition.Add(currentPartition)
				End If

				currentPartition.Add(diagram.Nodes(i))
			Next

			layout.CustomPartition = partition
		End If

		If colorPartitionsCheckBox.IsChecked = True AndAlso customRadioButton.IsChecked = True Then
			' LightSteelBlue
			' Turquoise
			Dim colors__1 As Color() = New Color() {Colors.Red, Colors.Yellow, Colors.Green, Colors.Blue, Colors.Orange, Colors.Purple, _
			 Color.FromArgb(&HFF, &HB0, &HC4, &HDE), Colors.Magenta, Colors.Black, Color.FromArgb(&HFF, &H40, &HE0, &HD0)}

			Dim p As Integer = 0
			For Each nodes As DiagramNodeCollection In partition
				For Each node As DiagramNode In nodes
					node.Brush = New SolidColorBrush(colors__1(p Mod colors__1.Length))
					node.Stroke = New SolidColorBrush(colors__1(p Mod colors__1.Length))
				Next

				p += 1
			Next
		End If

		Dim size As Integer = 32
		Dim temp As Integer
		If Integer.TryParse(subgraphSizeTextBox.Text, temp) Then
			size = temp
		End If
		layout.DesiredSubgraphSize = size

		layout.Arrange(diagram)
	End Sub

	Private Function OnCustomLayeredLayout(ByVal diagram As Diagram, ByVal items As IEnumerable) As Boolean
		Dim diagramItems = New DiagramItemCollection()
		For Each item As DiagramItem In items
			diagramItems.Add(item)
		Next

		Dim subgraphLayout As New LayeredLayout()
		subgraphLayout.Orientation = MindFusion.Diagramming.Wpf.Layout.Orientation.Vertical
		subgraphLayout.IgnoreNodeSize = False
		subgraphLayout.StraightenLongLinks = False
		Return subgraphLayout.Arrange(diagram, diagramItems)
	End Function

	Private Function OnCustomAnnealLayout(ByVal diagram As Diagram, ByVal items As IEnumerable) As Boolean
		Dim diagramItems = New DiagramItemCollection()
		For Each item As DiagramItem In items
			diagramItems.Add(item)
		Next

		Dim subgraphLayout As New AnnealLayout()
		Return subgraphLayout.Arrange(diagram, diagramItems)
	End Function

	Private Function OnCustomOrthogonalLayout(ByVal diagram As Diagram, ByVal items As IEnumerable) As Boolean
		Dim diagramItems = New DiagramItemCollection()
		For Each item As DiagramItem In items
			diagramItems.Add(item)
		Next
		Dim subgraphLayout As New OrthogonalLayout()
		Return subgraphLayout.Arrange(diagram, diagramItems)
	End Function


	Private Sub OnSubtreeLayoutNode(ByVal node As Object, ByVal oldBounds As Rect)
	End Sub

	Private Sub OnSubtreeLayoutLink(ByVal linkOrDataItem As Object)
		Dim link = TryCast(linkOrDataItem, DiagramLink)
		If link Is Nothing Then
			Return
		End If

		If customRadioButton.IsChecked = True AndAlso colorPartitionsCheckBox.IsChecked = True Then
			link.Stroke = link.Origin.Brush
			link.Brush = link.Origin.Brush
		End If
	End Sub

	Private Sub OnSubgraphLayoutNode(ByVal node As Object, ByVal oldBounds As Rect)
	End Sub

	Private Sub OnSubgraphLayoutLink(ByVal linkOrDataItem As Object)
		Dim link = TryCast(linkOrDataItem, DiagramLink)
		If link Is Nothing Then
			Return
		End If

		If customRadioButton.IsChecked = True AndAlso colorPartitionsCheckBox.IsChecked = True Then
			link.Stroke = link.Origin.Brush
			link.Brush = link.Origin.Brush
		End If
	End Sub

	Private Sub OnMasterLayoutNode(ByVal nodeOrDataItem As Object, ByVal oldBounds As Rect)
		Dim node = TryCast(nodeOrDataItem, DiagramNode)
		If node Is Nothing Then
			Return
		End If

		node.Stroke = Brushes.Green
	End Sub

	Private Sub OnMasterLayoutLink(ByVal linkOrDataItem As Object)
		Dim link = TryCast(linkOrDataItem, DiagramLink)
		If link Is Nothing Then
			Return
		End If

		link.Stroke = Brushes.Green
	End Sub

	Private Sub CreateBlock(ByVal numNodes As Integer, ByVal extraLinks As Integer, ByVal frame As BlockFrame, ByVal label As String)
		CreateBlock(numNodes, extraLinks, frame, label, 0)
	End Sub

	Private Sub CreateBlock(ByVal numNodes As Integer, ByVal extraLinks As Integer, ByVal frame As BlockFrame, ByVal label As String, ByVal extraNodes As Integer)
		Dim nodes As DiagramNodeCollection = diagram.Nodes
		Dim startIndex As Integer = nodes.Count

		For n As Integer = 0 To numNodes - 1
			Dim node As ShapeNode = diagram.Factory.CreateShapeNode(0, 0, 30, 30)
			node.Text = label
		Next

		If frame = BlockFrame.Cycle Then
			For l As Integer = 0 To numNodes - 1
				diagram.Factory.CreateDiagramLink(nodes(startIndex + l), nodes(startIndex + (l + 1) Mod numNodes))
			Next
		End If
		If frame = BlockFrame.Forest Then
			For l As Integer = startIndex + 1 To startIndex + (numNodes - 1)
				diagram.Factory.CreateDiagramLink(nodes(l), nodes(((l - 1) \ 10) * 10))
			Next
		End If
		If frame = BlockFrame.Star Then
			For l As Integer = startIndex To startIndex + (numNodes - 1)
				If l <> startIndex + numNodes \ 2 Then
					diagram.Factory.CreateDiagramLink(nodes(l), nodes(startIndex + numNodes \ 2))
				End If
			Next
		End If

		Dim random As New Random(1)
		For l As Integer = 1 To extraLinks - 1
			Dim s As Integer = random.[Next](startIndex, startIndex + numNodes)
			Dim t As Integer = random.[Next](startIndex, startIndex + numNodes)
			While s = t OrElse Connected(nodes(s), nodes(t))
				t = random.[Next](startIndex, startIndex + numNodes)
			End While
			diagram.Factory.CreateDiagramLink(nodes(s), nodes(t))
		Next

		For n As Integer = 0 To extraNodes - 1
			Dim node As ShapeNode = diagram.Factory.CreateShapeNode(0, 0, 5, 5)
			node.Text = label

			Dim s As Integer = random.[Next](startIndex, diagram.Nodes.Count - 1)
			Dim t As Integer = random.[Next](startIndex, diagram.Nodes.Count - 1)
			While s = t
				t = random.[Next](startIndex, diagram.Nodes.Count - 1)
			End While
			diagram.Factory.CreateDiagramLink(node, nodes(s))
			diagram.Factory.CreateDiagramLink(node, nodes(t))
		Next
	End Sub

	Private Function Connected(ByVal node1 As DiagramNode, ByVal node2 As DiagramNode) As Boolean
		Return GetCommonLinks(node1, node2).Count > 0
	End Function

	Private Function GetCommonLinks(ByVal node1 As DiagramNode, ByVal node2 As DiagramNode) As DiagramLinkCollection
		Dim commonLinks As New DiagramLinkCollection()

		For Each link As DiagramLink In node1.OutgoingLinks
			If link.Destination Is node2 Then
				commonLinks.Add(link)
			End If
		Next

		For Each link As DiagramLink In node1.IncomingLinks
			If link.Origin Is node2 Then
				commonLinks.Add(link)
			End If
		Next

		Return commonLinks
	End Function

	Private Sub saveButton_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
		Dim dialog As New SaveFileDialog()
		dialog.Filter = "*.xml|*.xml||"
		If dialog.ShowDialog() = True Then
			diagram.SaveToXml(dialog.FileName)
		End If
	End Sub

	Private Sub loadButton_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
		Dim dialog As New OpenFileDialog()
		dialog.Filter = "*.xml|*.xml||"
		If dialog.ShowDialog() = True Then
			diagram.LoadFromXml(dialog.FileName)
		End If
	End Sub

	Private Sub clearButton_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
		diagram.ClearAll()
	End Sub

End Class

Enum BlockFrame

	Cycle
	Forest
	Star

End Enum
